package org.antlr.analysis;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import org.antlr.analysis.DFA;
import org.antlr.analysis.DFAState;
import org.antlr.analysis.Label;
import org.antlr.analysis.LookaheadSet;
import org.antlr.analysis.NFAState;
import org.antlr.analysis.PredicateLabel;
import org.antlr.analysis.SemanticContext;
import org.antlr.misc.IntervalSet;
import org.antlr.misc.MultiMap;

public class LL1DFA extends DFA {
  public LL1DFA(int decisionNumber, NFAState decisionStartState, LookaheadSet[] altLook) {
    DFAState s0 = newState();
    this.startState = s0;
    this.nfa = decisionStartState.nfa;
    this.nAlts = this.nfa.grammar.getNumberOfAltsForDecisionNFA(decisionStartState);
    this.decisionNumber = decisionNumber;
    this.decisionNFAStartState = decisionStartState;
    initAltRelatedInfo();
    this.unreachableAlts = null;
    for (int alt = 1; alt < altLook.length; alt++) {
      DFAState acceptAltState = newState();
      acceptAltState.acceptState = true;
      setAcceptState(alt, acceptAltState);
      acceptAltState.k = 1;
      acceptAltState.cachedUniquelyPredicatedAlt = alt;
      Label e = getLabelForSet((altLook[alt]).tokenTypeSet);
      s0.addTransition(acceptAltState, e);
    } 
  }
  
  public LL1DFA(int decisionNumber, NFAState decisionStartState, MultiMap<IntervalSet, Integer> edgeMap) {
    DFAState s0 = newState();
    this.startState = s0;
    this.nfa = decisionStartState.nfa;
    this.nAlts = this.nfa.grammar.getNumberOfAltsForDecisionNFA(decisionStartState);
    this.decisionNumber = decisionNumber;
    this.decisionNFAStartState = decisionStartState;
    initAltRelatedInfo();
    this.unreachableAlts = null;
    for (Map.Entry<IntervalSet, List<Integer>> entry : edgeMap.entrySet()) {
      IntervalSet edge = entry.getKey();
      List<Integer> alts = entry.getValue();
      Collections.sort(alts);
      DFAState s = newState();
      s.k = 1;
      Label e = getLabelForSet(edge);
      s0.addTransition(s, e);
      if (alts.size() == 1) {
        s.acceptState = true;
        int alt = ((Integer)alts.get(0)).intValue();
        setAcceptState(alt, s);
        s.cachedUniquelyPredicatedAlt = alt;
        continue;
      } 
      s.resolvedWithPredicates = true;
      for (int i = 0; i < alts.size(); i++) {
        int alt = ((Integer)alts.get(i)).intValue();
        s.cachedUniquelyPredicatedAlt = -1;
        DFAState predDFATarget = getAcceptState(alt);
        if (predDFATarget == null) {
          predDFATarget = newState();
          predDFATarget.acceptState = true;
          predDFATarget.cachedUniquelyPredicatedAlt = alt;
          setAcceptState(alt, predDFATarget);
        } 
        SemanticContext.Predicate synpred = getSynPredForAlt(decisionStartState, alt);
        if (synpred == null)
          synpred = new SemanticContext.TruePredicate(); 
        s.addTransition(predDFATarget, new PredicateLabel(synpred));
      } 
    } 
  }
  
  protected Label getLabelForSet(IntervalSet edgeSet) {
    Label e;
    int atom = edgeSet.getSingleElement();
    if (atom != -7) {
      e = new Label(atom);
    } else {
      e = new Label(edgeSet);
    } 
    return e;
  }
  
  protected SemanticContext.Predicate getSynPredForAlt(NFAState decisionStartState, int alt) {
    int walkAlt = decisionStartState.translateDisplayAltToWalkAlt(alt);
    NFAState altLeftEdge = this.nfa.grammar.getNFAStateForAltOfDecision(decisionStartState, walkAlt);
    NFAState altStartState = (NFAState)(altLeftEdge.transition[0]).target;
    if (altStartState.transition[0].isSemanticPredicate()) {
      SemanticContext ctx = (altStartState.transition[0]).label.getSemanticContext();
      if (ctx.isSyntacticPredicate()) {
        SemanticContext.Predicate p = (SemanticContext.Predicate)ctx;
        if (p.predicateAST.getType() == 14) {
          if (ctx.isSyntacticPredicate())
            this.nfa.grammar.synPredUsedInDFA(this, ctx); 
          return (SemanticContext.Predicate)(altStartState.transition[0]).label.getSemanticContext();
        } 
      } 
    } 
    return null;
  }
}
